/****************************************************
* COKE MACHINE SIMULATION WITH AUTOMATA THEORY      *
* LINGUAGENS FORMAIS E AUTOMATOS                    *
* Prof.: Jones Oliveira Albuquerque                 *
*                                                   *
* INTEGRANTES:                                      *
* Anderson de Rezende Rocha [undersun@comp.ufla.br] *
* Flvio Luiz Alves         [alves@comp.ufla.br]    *
* Jlio Csar Alves         [jcalves@comp.ufla.br]  *
* Rodrigo Nazar Leite      [canniggia@comp.ufla.br]*
*                                                   *
* Bacharelado em Cincia da Computao              *
* Universidade Federal de Lavras, MG, Brasil        *
****************************************************/

// Esta classe define um automato finito deterministico qualquer, com
// mensagem de saida no estado (Maquina de Moore)
// Ela eh template em relacao ao alfabeto, ou seja, o alfabeto
// pode ser qualquer classe definida pelo usuario desta classe

#ifndef AUTOMATO_H
#define AUTOMATO_H

#include "estado.h"

// OBSERVAO
// Depois de inserir todos os estados e producoes se for utilizada a funcao ProcessarSimbolo()
// o usurio eh responsavel por Reinicializar o Automato antes de passar o 1o. simbolo

// Definio da classe clAutomato como template
template <class TSimbolo>
class clAutomato
{
    public:
        // Construtor que recebe um identificador para o noh, um booleano [verdadeiro/falso]
        // identificando se este e final ou nao e um char que ser a mensagem de saida d5o primeiro
        // noh do automato. Esta mensagem fica no noh pois usamos a abordagem das maquinas
        // de moore.
        clAutomato<TSimbolo>(int idEstadoInicial, bool final, char saida[36]);
        // Destrutor padrao para desalocar memria
        ~clAutomato<TSimbolo>();
        // Insere um novo estado no automato, recebe um identificador numrico, um booleano
        // dizendo se o noh  final e a saida que este estado ter
        bool InserirEstado(int idEstado, bool final, char saida[36]);
        // Insere uma producao a partir de um estado de inicio e um estado de destino, mais
        // o simbolo especifico para esta producao
        bool InserirProducao(int idEstadoPartida, int idEstadoDestino, TSimbolo* simbolo);
        // Processa uma palavra inteira de entrada de uma vez
        bool ProcessarPalavra(clLista<TSimbolo*> palavra);
        // Processa um simbolo da entrada de cada vez
        // o boolean do parametro serve para informar ao automato que a fita acabou
        bool ProcessarSimbolo(TSimbolo *simbolo, char saida[36], bool ehUltimoSimbolo = false);
        // Retorna se a palavra foi ou nao aceita
        bool getPalavraFoiAceita();
        // Reinicializa o automato para processamento de uma nova palavra
        void ReinicializarAutomato();

    private:
        // Encontra um estado especifico para, por exemplo, ser usado pela
        // funcao que insere uma producao
        clEstado<TSimbolo>* EncontrarEstado(int id);
        // Guarda qual o estado inicial do automato
        clEstado<TSimbolo>* estadoInicial;
        // Estado atual de processamento de uma palavra em especifico
        clEstado<TSimbolo>* estadoAtual;
        // Guarda se a palavra esta sendo ou nao aceita at o momento
        bool palavraFoiAceita;
};

// Construtor padrao
template <class TSimbolo>
clAutomato<TSimbolo>::clAutomato<TSimbolo>(int idEstadoInicial, bool final, char saida[36])
{
    estadoInicial = new clEstado<TSimbolo>(idEstadoInicial, final, saida);
    estadoAtual = estadoInicial;
}

// Destrutor, enquanto houver membros alocados dinamicamente desaloque-os
template <class TSimbolo>
clAutomato<TSimbolo>::~clAutomato<TSimbolo>()
{
    clEstado<TSimbolo> *estado;
    estado = estadoInicial;
    while (estadoAtual != NULL)
    {
        estado = estadoAtual;
        estadoAtual = estado->getProximoEstado();
        delete estado;
    }
}

// Insere um estado. A partir do idEstado se este ja existir o identificador esta
// duplicado, caso contrario, insira-o no automato.
template <class TSimbolo>
bool clAutomato<TSimbolo>::InserirEstado(int idEstado, bool final, char saida[36])
{
    if (EncontrarEstado(idEstado))
        return false;
    else
    {
        clEstado<TSimbolo> *estado;
        estado = new clEstado<TSimbolo>(idEstado, final, saida);
        estadoAtual->setProximoEstado(estado);
        estadoAtual = estado;
    }
    return true;
}

// Insere uma producao. Busca se o estado que contem o id de partida e o estado
// que contem o id de destino. Retorne-os. Se forem diferentes de NULL entao
// ligue-os insira uma producao com o simbolo passado, entre eles.
template <class TSimbolo>
bool clAutomato<TSimbolo>::InserirProducao(int idEstadoPartida, int idEstadoDestino,
                                                                    TSimbolo* simbolo)
{
    clEstado<TSimbolo> *estadoPartida = EncontrarEstado(idEstadoPartida);
    clEstado<TSimbolo> *estadoChegada = EncontrarEstado(idEstadoDestino);
    if (estadoPartida == NULL || estadoChegada == NULL)
        return false;
    else
        return (estadoPartida->InserirProducao(simbolo, estadoChegada));
}

// Retorna se uma palavra foi aceita ou nao
template <class TSimbolo>
bool clAutomato<TSimbolo>::getPalavraFoiAceita()
{
    return palavraFoiAceita;
}

// Reinicializa o automato para processamento de outra palavra qualquer
template <class TSimbolo>
void clAutomato<TSimbolo>::ReinicializarAutomato()
{
    estadoAtual = estadoInicial;
    palavraFoiAceita = false;
}

// Funcao delta-estendida, processa uma palavra inteira de uma vez
// Por definio apenas aplica a funcao delta comun "ProcessarSimbolo"
// varias vezes. Na verdade o numero de vezes eh a cardinalidade da palavra
// de entrada.
template <class TSimbolo>
bool clAutomato<TSimbolo>::ProcessarPalavra(clLista<TSimbolo*> palavra)
{
    ReinicializarAutomato();
    clNohLista<TSimbolo*> *simbolo;
    simbolo = palavra.Cabeca();
    while(estadoAtual != NULL && simbolo != NULL)
    {
        ProcessarSimbolo(simbolo->TValor());
        simbolo = simbolo->Proximo();
    }

    if (estadoAtual != NULL)
        palavraFoiAceita = estadoAtual->getEhFinal();
    else
        palavraFoiAceita = false;
    return (getPalavraFoiAceita());
}

// O usuario deve reinizializar o automato quando necessario
// Retorna verdadeiro se ha producao para tal simbolo
template <class TSimbolo>
bool clAutomato<TSimbolo>::ProcessarSimbolo(TSimbolo *simbolo, char saida[36],
                                                              bool ehUltimoSimbolo)
{
    estadoAtual = estadoAtual->Consumir(simbolo);
    if (ehUltimoSimbolo && estadoAtual != NULL && estadoAtual->getEhFinal())
    {
        palavraFoiAceita = true;
        strcpy(saida,estadoAtual->getSaida());
        return getPalavraFoiAceita();
    }
    else if (estadoAtual != NULL)
    {
        if (estadoAtual != NULL)
            strcpy(saida,estadoAtual->getSaida());
        return true;
    }
    return false;
}

// Encontra um estado em especial a partir de seu identificador
// Retorna NULL se o estado no foi encontrado
template <class TSimbolo>
clEstado<TSimbolo>*  clAutomato<TSimbolo>::EncontrarEstado(int id)
{
    clEstado<TSimbolo>* estado;
    estado = estadoInicial;
    while (estado != NULL)
    {
        if (id == estado->getId())
            return estado;
        else
            estado = estado->getProximoEstado();
    }
    return estado;
}

#endif
