/****************************************************
* COKE MACHINE SIMULATION WITH AUTOMATA THEORY      *
* LINGUAGENS FORMAIS E AUTOMATOS                    *
* Prof.: Jones Oliveira Albuquerque                 *
*                                                   *
* INTEGRANTES:                                      *
* Anderson de Rezende Rocha [undersun@comp.ufla.br] *
* Flvio Luiz Alves         [alves@comp.ufla.br]    *
* Jlio Csar Alves         [jcalves@comp.ufla.br]  *
* Rodrigo Nazar Leite      [canniggia@comp.ufla.br]*
*                                                   *
* Bacharelado em Cincia da Computao              *
* Universidade Federal de Lavras, MG, Brasil        *
****************************************************/

// includes noramis da wxWindows
// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

// nossos includes
#include "wx/image.h" // para trabalhar com as imagens
#include "wx/file.h" // para trabalhar com os arquivos de imagens
#include "automato.h" // classe clAutomato, define um automato
#include "simbolo.h" // calsse clSimbolo, define o alfabeto do automato da coke Machine
#include "wx/textfile.h" // para trabalhar com a base de dados (producoes.bda e estados.bda)

class cokeMachineFrame;
class cokeMachineApp;

// Define a janela interna do programa
class cokeMachineCanvas: public wxScrolledWindow
{
    public:
    cokeMachineCanvas() {};
    // construtor que desenha a janela interna do programa (com botes e etc)
    cokeMachineCanvas( wxWindow *parent, wxWindowID, const wxPoint &pos, const wxSize &size );
    ~cokeMachineCanvas();
    // desenha as figuras
    void OnPaint( wxPaintEvent &event );
    //          AES DOS BOTES
    // pede um ngainana
    void OnPedirIngainana( wxCommandEvent &event );
    // pede um macaia-cola
    void OnPedirMacaia_Cola( wxCommandEvent &event );
    // pede uma oranjaci
    void OnPedirOranjaci( wxCommandEvent &event );
    // insere uma moeda de R$ 0,05
    void OnInserirCinco( wxCommandEvent &event );
    // insere uma moeda de R$ 0,010
    void OnInserirDez( wxCommandEvent &event );
    // insere uma moeda de R$ 0,25
    void OnInserirVinteCinco( wxCommandEvent &event );
    // insere uma moeda de R$ 0,50
    void OnInserirCinquenta( wxCommandEvent &event );
    // insere uma moeda de R$ 1,00
    void OnInserirUmReal( wxCommandEvent &event );
    // pede o troco
    void OnPedirTroco( wxCommandEvent &event );
    // janela sobre
    void OnSobre( wxCommandEvent &event );
    // cria o automato a partir da base de dados
    clAutomato<clSimbolo> * CriarAutomato();

    // display que mostra as saidas da maquina de moore
    wxTextCtrl *display;
    // texto com os precos
    wxStaticText *precos;
    // figuras utilizadas
    wxBitmap  *ingainana_gif;
    wxBitmap  *macaia_cola_gif;
    wxBitmap  *oranjaci_gif;
    // automato propriamente dito
    clAutomato<clSimbolo> *automato;
    // variavel auxiliar para as saidas da maquina de moore
    char saida[36];
    // variavel auxiliar para os simbolos do alfabeto da coke machine
    clSimbolo simbolo;

private:
    DECLARE_DYNAMIC_CLASS(cokeMachineCanvas)
    DECLARE_EVENT_TABLE()
};

// janela pricipal do programa
class cokeMachineFrame: public wxFrame
{
public:
    cokeMachineFrame();

    // referencia para a janela interna do programa
    cokeMachineCanvas *f_canvas;

private:
    DECLARE_DYNAMIC_CLASS(cokeMachineFrame)
    DECLARE_EVENT_TABLE()
};

// inicia a aplicacao
class cokeMachineApp : public wxApp
{
    virtual bool OnInit();
};

// main program

IMPLEMENT_APP(cokeMachineApp)

// identificadores
enum {ID_PEDIR_INGAINANA = 109, ID_PEDIR_MACAIA_COLA, ID_PEDIR_ORANJACI,
            ID_INSERIR_CINCO, ID_INSERIR_DEZ, ID_INSERIR_CINQUENTA,
            ID_INSERIR_VINTE_CINCO, ID_INSERIR_UM_REAL, ID_PEDIR_TROCO, ID_B_SOBRE,};

IMPLEMENT_DYNAMIC_CLASS(cokeMachineCanvas, wxScrolledWindow)

BEGIN_EVENT_TABLE(cokeMachineCanvas, wxScrolledWindow)
  EVT_PAINT(cokeMachineCanvas::OnPaint)
  EVT_BUTTON( ID_PEDIR_INGAINANA,   cokeMachineCanvas::OnPedirIngainana)
  EVT_BUTTON( ID_PEDIR_MACAIA_COLA,   cokeMachineCanvas::OnPedirMacaia_Cola)
  EVT_BUTTON( ID_PEDIR_ORANJACI,   cokeMachineCanvas::OnPedirOranjaci)

  EVT_BUTTON( ID_INSERIR_CINCO, cokeMachineCanvas::OnInserirCinco)
  EVT_BUTTON( ID_INSERIR_DEZ, cokeMachineCanvas::OnInserirDez)
  EVT_BUTTON( ID_INSERIR_VINTE_CINCO, cokeMachineCanvas::OnInserirVinteCinco)
  EVT_BUTTON( ID_INSERIR_CINQUENTA, cokeMachineCanvas::OnInserirCinquenta)
  EVT_BUTTON( ID_INSERIR_UM_REAL, cokeMachineCanvas::OnInserirUmReal)
  EVT_BUTTON( ID_PEDIR_TROCO, cokeMachineCanvas::OnPedirTroco)
  EVT_BUTTON( ID_B_SOBRE, cokeMachineCanvas::OnSobre)
END_EVENT_TABLE()


cokeMachineCanvas::cokeMachineCanvas( wxWindow *parent, wxWindowID id,
                    const wxPoint &pos, const wxSize &size )
        : wxScrolledWindow( parent, id, pos, size, wxSUNKEN_BORDER )
{
    // o automato eh criado a partir da base de dados
    automato = CriarAutomato();
    if (automato != NULL)
    {
        SetBackgroundColour(wxColour(194, 194, 194));

        // criando botoes
        (void) new wxButton(this, ID_PEDIR_INGAINANA, "IngaNan", wxPoint(30,145),wxSize(90, 25));
        (void) new wxButton(this, ID_PEDIR_MACAIA_COLA, "Macaia-Cola", wxPoint(30,280),wxSize(90, 25));
        (void)new wxButton(this,ID_PEDIR_ORANJACI,"OranJaci",wxPoint(150,145),wxSize(90, 25));

        (void) new wxButton(this, ID_INSERIR_CINCO, "R$ 0,05", wxPoint(150,205), wxSize(90, 25));
        (void) new wxButton(this, ID_INSERIR_DEZ, "R$ 0,10", wxPoint(150,245), wxSize(90, 25));
        (void) new wxButton(this, ID_INSERIR_VINTE_CINCO, "R$ 0,25", wxPoint(150,285), wxSize(90, 25));
        (void) new wxButton(this, ID_INSERIR_CINQUENTA, "R$ 0,50", wxPoint(150,325), wxSize(90, 25));
        (void) new wxButton(this, ID_INSERIR_UM_REAL, "R$ 1,00", wxPoint(150,365), wxSize(90, 25));
        (void) new wxButton(this, ID_PEDIR_TROCO, "Troco", wxPoint(150,405), wxSize(90, 25));
        (void) new wxButton(this, ID_B_SOBRE, "Sobre", wxPoint(30,330), wxSize(90, 25));

        // criando display para mostrar as saidas da maquina de moore
         display = new wxTextCtrl(this, 120, "Insira Moedas", wxPoint(30, 10), wxSize(210, 25),
                                                        wxTE_READONLY);
         // criando texto com os precos dos refrigerantess
        precos =new wxStaticText(this, 121,"IngaNan....... R$ 0,85\nMacaia-Cola. "
                "R$ 1,00\nOranJaci......... R$ 0,75", wxPoint(10,380));

        // carregando as figuras
        ingainana_gif = (wxBitmap*) NULL;
        macaia_cola_gif  = (wxBitmap*) NULL;
        oranjaci_gif = (wxBitmap*) NULL;

        wxBitmap bitmap( 100, 100 );
        wxImage image( bitmap );

#if wxUSE_GIF
        image.Destroy();
        if ( !image.LoadFile(wxString("ingainana.gif")) )
            wxLogError("No foi possvel carregar \"ingainana.gif\"");
        else
            ingainana_gif = new wxBitmap( image.ConvertToBitmap() );

        image.Destroy();
        if ( !image.LoadFile(wxString("macaia_cola.gif")))
            wxLogError("No foi possvel carregar \"macaia_cola.gif\"");
        else
            macaia_cola_gif = new wxBitmap( image.ConvertToBitmap() );

        image.Destroy();
        if ( !image.LoadFile(wxString("oranjaci.gif")))
            wxLogError("No foi possvel carregar \"oranjaci.gif\"");
        else
            oranjaci_gif = new wxBitmap( image.ConvertToBitmap() );
#endif
    }
    else
        wxLogError("No foi possvel criar o automato. Verifique a base de dados");
}

// destroi a coke machine, liberando a memoria alocada
cokeMachineCanvas::~cokeMachineCanvas()
{
    delete ingainana_gif;
    delete macaia_cola_gif;
    delete oranjaci_gif;
    delete display;
    delete automato;
}

// desenha as figuras
void cokeMachineCanvas::OnPaint( wxPaintEvent &WXUNUSED(event) )
{
    wxPaintDC dc( this );
    PrepareDC( dc );

    if (ingainana_gif && ingainana_gif->Ok())
        dc.DrawBitmap( *ingainana_gif, 45, 40 );

    if (macaia_cola_gif && macaia_cola_gif->Ok())
        dc.DrawBitmap( *macaia_cola_gif, 45, 175 );

    if (macaia_cola_gif && macaia_cola_gif->Ok())
        dc.DrawBitmap( *oranjaci_gif, 165, 40);
}

// O comentarios desta funcao serve para as duas subsequentes,
// o que muda sao os refrigerantes e, consequentemente os simbolos usados

// pede um ingainana
void cokeMachineCanvas::OnPedirIngainana( wxCommandEvent &WXUNUSED(event) )
{
    // simbolo do alfabeto que representa a acao de pedir um ingainana
    simbolo = PEDIR_INGAINANA;
    // processa o simbolo no automato, e recupera a saida da maquina de moore
    automato->ProcessarSimbolo(&simbolo, saida);
    if (saida[0] != 'F') // se a maquina tem o dinheiro suficiente ...
    // obs: quando nao tem dinheiro suficiente a mensagem da saida eh "Falta dinheiro...", por isso o 'F'
    {
        // caixa de mensagens dizendo para o usuario pegar o refrigerante
        wxMessageBox( saida, "IngaNan", wxOK);
        // simbolo do alfabeto que representa a acao de pegar um refrigerante
        simbolo = PEGAR_REFRIGERANTE;
        // processa o simbolo no automato, e recupera a saida da maquina de moore
        automato->ProcessarSimbolo(&simbolo, saida);
        if (saida[0] == 'I') // Se nao ha troco...
            automato->ReinicializarAutomato(); // ... o autmato eh reinicializado
        // quando nao ha troco a mensagem de saida eh "Insira Moedas", por isso o 'I'

    }
    // mostra a saida da maquina de moore no display
    display->Clear();
    (*display) << saida;
}

void cokeMachineCanvas::OnPedirMacaia_Cola( wxCommandEvent &WXUNUSED(event) )
{
    simbolo = PEDIR_MACAIA_COLA;
    automato->ProcessarSimbolo(&simbolo, saida);
    if (saida[0] != 'F')
    {
        wxMessageBox( saida, "Macaia-Cola", wxOK);
        simbolo = PEGAR_REFRIGERANTE;
        automato->ProcessarSimbolo(&simbolo, saida);
        if (saida[0] == 'I')
            automato->ReinicializarAutomato();
    }
    display->Clear();
    (*display) << saida;
}

void cokeMachineCanvas::OnPedirOranjaci( wxCommandEvent &WXUNUSED(event) )
{
    simbolo = PEDIR_ORANJACI;
    automato->ProcessarSimbolo(&simbolo, saida);
    if (saida[0] != 'F')
    {
        wxMessageBox( saida, "OranJaci", wxOK);
        simbolo = PEGAR_REFRIGERANTE;
        automato->ProcessarSimbolo(&simbolo, saida);
        if (saida[0] == 'I')
            automato->ReinicializarAutomato();
    }
    display->Clear();
    (*display) << saida;
}


// O comentarios desta funcao serve para as quatro subsequentes,
// o que muda eh o valor de cada moeda e, consequentemente os simbolos usados

// insere R$ 0,05 na maquina
void cokeMachineCanvas::OnInserirCinco( wxCommandEvent &WXUNUSED(event) )
{
    // simbolo do alfabeto que representa a acao de inserir uma moeda de R$ 0,05
    simbolo = R$005;
    // processa o simbolo no automato, e recupera a saida da maquina de moore
    automato->ProcessarSimbolo(&simbolo, saida);
    // mostra a saida recuperada no display
    display->Clear();
    (*display) << saida;
}

void cokeMachineCanvas::OnInserirDez( wxCommandEvent &WXUNUSED(event) )
{
    simbolo = R$010;
    automato->ProcessarSimbolo(&simbolo, saida);
    display->Clear();
    (*display) << saida;
}

void cokeMachineCanvas::OnInserirVinteCinco( wxCommandEvent &WXUNUSED(event) )
{
    simbolo = R$025;
    automato->ProcessarSimbolo(&simbolo, saida);
    display->Clear();
    (*display) << saida;
}

void cokeMachineCanvas::OnInserirCinquenta( wxCommandEvent &WXUNUSED(event) )
{
    simbolo = R$050;
    automato->ProcessarSimbolo(&simbolo, saida);
    display->Clear();
    (*display) << saida;
}

void cokeMachineCanvas::OnInserirUmReal( wxCommandEvent &WXUNUSED(event) )
{
    simbolo = R$100;
    automato->ProcessarSimbolo(&simbolo, saida);
    display->Clear();
    (*display) << saida;
}

// pede o troco
void cokeMachineCanvas::OnPedirTroco( wxCommandEvent &WXUNUSED(event) )
{
    // simbolo do alfabeto que representa a acao de inserir uma moeda de R$ 0,05
    simbolo = PEDIR_TROCO;
    // processa o simbolo no automato, e recupera a saida da maquina de moore
    automato->ProcessarSimbolo(&simbolo, saida);
    // exibe uma mensagem pra o usuario pegar o troco
    wxMessageBox( saida, "Troco", wxOK);
    // reinicializa o automato
    automato->ReinicializarAutomato();
    // mostra a saida recuperada no display
    display->Clear();
    (*display) << "Insira Moedas";
}

// janela sobre
void cokeMachineCanvas::OnSobre( wxCommandEvent &WXUNUSED(event) )
{
    wxMessageBox("UNIVERSIDADE FEDERAL DE LAVRAS\n"
                                "CINCIA DA COMPUTAO\n\n"
                                "Coke Machine Wtih Automata Theory\n\n"
                                "Desenvolvido por:\n    Anderson de Resende Rocha\n"
                                "    Flvio Luis Alves\n    Jlio Csar Alves\n"
                                "    Rodrigo Nazar da Silva Leite\n\n"
                                "{undersun,alves,jcalves,canniggia}@comp.ufla.br\n\n"
                                "Desenvolvido como trabalho para a disciplina de\n"
                                "Linguagens Formais e Autmatos\n"
                                "Professor: Jones de Oliveira Albuquerque\n",
                                "Sobre o Refrigereco Machine...", wxOK);
}

// Cria o automato a partir da base de dados
clAutomato<clSimbolo> *cokeMachineCanvas::CriarAutomato()
{
    // referencia para um automato
    clAutomato<clSimbolo> *automato;
    // objeto usado para se trabalhar com os arquivos da base de dados
    wxTextFile arquivo;
    // variavel auxiliar
    wxString str;

    long int identificador;
    bool ehFinal;
    long int final;
    char saida[36];

    // se os arquivos foram abertos com exito cria o automato a partir deles...

    if (!arquivo.Open((const wxString)"estados.bda"))
    {
        wxMessageBox("No foi possvel criar ou abrir arquivo estados.bda",
                                        "Erro de leitura", wxOK | wxICON_ERROR);
        return NULL;
    }
    else
    {
        // inserindo estados

        // str recebe a primeira linha de dados validos (linhas nao vazias e sem comentrios)
        str = arquivo.GetFirstLine();
        
        while ((str.IsEmpty() || str.GetChar(0) == '/') && !arquivo.Eof())
            str = arquivo.GetNextLine();

        if (!arquivo.Eof())
        {
            (str.BeforeFirst(' ')).ToLong(&identificador);
            str = str.AfterFirst(' ');
            (str.BeforeFirst(' ')).ToLong(&final);
            ehFinal = (bool)final;
            str = str.AfterFirst(' ');
            strcpy(saida, str.GetData());
            // cria automato com o estado inicial
            automato = new clAutomato<clSimbolo>((int)identificador, ehFinal, saida);

            for ( str = arquivo.GetNextLine(); !arquivo.Eof(); str = arquivo.GetNextLine() )
            {
                while ((str.IsEmpty() || str.GetChar(0) == '/') && !arquivo.Eof())
                    str = arquivo.GetNextLine();
                if (!arquivo.Eof())
                {
                    (str.BeforeFirst(' ')).ToLong(&identificador);
                    str = str.AfterFirst(' ');
                    ehFinal = (bool)((str.BeforeFirst(' ')).ToLong(&final));
                    ehFinal = (bool)final;
                    str = str.AfterFirst(' ');
                    strcpy(saida, str.GetData());

                    // insere estado
                    if (!automato->InserirEstado((int)identificador, ehFinal, saida))
                        return NULL;
                }
                else
                    str = arquivo.GetPrevLine();
            }
            if (!str.IsEmpty())
            {
                (str.BeforeFirst(' ')).ToLong(&identificador);
                str = str.AfterFirst(' ');
                ehFinal = (bool)((str.BeforeFirst(' ')).ToLong(&final));
                ehFinal = (bool)final;
                str = str.AfterFirst(' ');
                strcpy(saida, str.GetData());

                // insere ultimo estado
                if (!automato->InserirEstado((int)identificador, ehFinal, saida))
                    return NULL;
            }
        }
    }
    arquivo.Close();

    int idPartida, idChegada;
    clSimbolo simbolo;

    if (!arquivo.Open((const wxString)"producoes.bda"))
    {
        wxMessageBox("No foi possvel criar ou abrir arquivo producoes.bda",
                                        "Erro de leitura", wxOK | wxICON_ERROR);
        return NULL;
    }
    else
    {
        // inserindo producoes

        for ( str = arquivo.GetFirstLine(); !arquivo.Eof(); str = arquivo.GetNextLine() )
        {
            while ((str.IsEmpty() || str.GetChar(0) == '/') && !arquivo.Eof())
                str = arquivo.GetNextLine();
            if (!arquivo.Eof())
            {
                (str.BeforeFirst(' ')).ToLong(&identificador);
                idPartida = (int)identificador;
                str = str.AfterFirst(' ');
                (str.BeforeFirst(' ')).ToLong(&identificador);
                simbolo = (int)identificador;
                (str.AfterFirst(' ')).ToLong(&identificador);
                idChegada = (int)identificador;

                // insere producao
                 if (!(automato->InserirProducao(idPartida, idChegada, &simbolo)))
                        return NULL;
            }
            else
                str = arquivo.GetPrevLine();
        }
        if (!str.IsEmpty())
        {
            (str.BeforeFirst(' ')).ToLong(&identificador);
            idPartida = (int)identificador;
            str = str.AfterFirst(' ');
            (str.BeforeFirst(' ')).ToLong(&identificador);
            simbolo = (int)identificador;
            (str.AfterFirst(' ')).ToLong(&identificador);
            idChegada = (int)identificador;

            // insere ultima producao
             if (!(automato->InserirProducao(idPartida, idChegada, &simbolo)))
                return NULL;
        }
    }
    arquivo.Close();
    automato->ReinicializarAutomato();
    return automato;
}

IMPLEMENT_DYNAMIC_CLASS( cokeMachineFrame, wxFrame )

BEGIN_EVENT_TABLE(cokeMachineFrame,wxFrame)
END_EVENT_TABLE()

// janela principal do programa
cokeMachineFrame::cokeMachineFrame()
       : wxFrame( (wxFrame *)NULL, -1, "Refrigereco Machine",
                  wxPoint(20,20), wxSize(270,480) )
{
    // janela interna do programa
    f_canvas = new cokeMachineCanvas( this, -1, wxPoint(0,0), wxSize(10,10) );
}

// incializa a janela principal
bool cokeMachineApp::OnInit()
{
#if wxUSE_GIF
  wxImage::AddHandler( new wxGIFHandler );
#endif

  wxFrame *frame = new cokeMachineFrame();
  frame->Centre();
  frame->Show( TRUE );

  return TRUE;
}
