/****************************************************
* COKE MACHINE SIMULATION WITH AUTOMATA THEORY      *
* LINGUAGENS FORMAIS E AUTOMATOS                    *
* Prof.: Jones Oliveira Albuquerque                 *
*                                                   *
* INTEGRANTES:                                      *
* Anderson de Rezende Rocha [undersun@comp.ufla.br] *
* Flvio Luiz Alves         [alves@comp.ufla.br]    *
* Jlio Csar Alves         [jcalves@comp.ufla.br]  *
* Rodrigo Nazar Leite      [canniggia@comp.ufla.br]*
*                                                   *
* Bacharelado em Cincia da Computao              *
* Universidade Federal de Lavras, MG, Brasil        *
****************************************************/

// classe que define uma lista simples

#ifndef CLLISTA_H
#define CLLISTA_H

#include <iostream>
#include "nohlista.h"

template <class TValor>
class clLista
{
    public:
        clLista();           
        clLista(const clLista<TValor> &lista); 
        ~clLista();           
        void clLista<TValor>::Limpar();
        clNohLista<TValor> *Primeiro() const;   
        void InserirNoFim(const TValor &val);   
        // encontra um dado e o atualiza
        bool Encontrar(TValor& valor);  
    protected:
        clNohLista<TValor> *primeiro;   
};


// Esta funo encontra um dado passado e o atualiza, por que isso?
// Isso permite, por exemplo, que a classe TValor defina que um objeto eh igual ao outro
// apenas em relao a um atributo. Deste modo eu posso querer encontrar, na lista, um objeto
// igual ao passado em relao a este atributo, mas com os outros atributos diferentes.
template <class TValor>
bool clLista<TValor>::Encontrar(TValor& valor)
{
    if (primeiro != NULL)
    {
        clNohLista<TValor> *nohAux;
        nohAux = primeiro;
        while (nohAux != NULL)
        {
            if (valor == nohAux->Valor())
            {
                valor = nohAux->Valor();
                return true;
            }
            else
                nohAux = nohAux->Proximo();
        }
    }
    return false;
}

template <class TValor>
clLista<TValor>::clLista()
{
    primeiro = NULL;
}

template <class TValor>
clLista<TValor>::clLista(const clLista<TValor> &lista)
{
    clNohLista<TValor> *noh;

    // criar a lista vazia...
    primeiro = NULL;
    noh = lista.Primeiro();

    // ... e copiar cada elemento
    while (noh != NULL)
    {
        InserirNoFim(noh->Valor());
        noh = noh->Proximo();
    }
}

template <class TValor>
void clLista<TValor>::Limpar()
// Apaga todos elementos da lista, tornando-a vazia
{
    clNohLista<TValor> *aux;

    // destruir todos os nohs da lista
    aux = primeiro;
    while(aux != NULL)
    {
        primeiro = aux->Proximo();
        delete aux;
        aux = primeiro;
    }
}


template <class TValor>
clLista<TValor>::~clLista()
{
    Limpar(); //remover todos os nohs
}

template <class TValor>
clNohLista<TValor> *clLista<TValor>::Primeiro() const
{
    return primeiro;
}

template <class TValor>
void clLista<TValor>::InserirNoFim(const TValor &val)
// Insere UMA COPIA do valor passado no fim da lista
{
    clNohLista<TValor> *atual;
    clNohLista<TValor> *anterior;

    atual = primeiro;
    anterior = NULL;

    while (atual != NULL)
    {
        anterior = atual;
        atual = atual->Proximo();
    }
    atual = new clNohLista<TValor>;
    atual->CopiarValor(val);
    if (anterior == NULL)
        primeiro = atual;
    else
        anterior->Proximo(atual);
}

#endif
